#include <dos.h>
#include <conio.h>
#include <stdio.h>


//  LPT1 register address
#define  DATA1           0x378
#define  STATUS1         DATA+1
#define  CONTROL1        DATA+2

//  LPT2 register address
#define  DATA2       0x278
#define  STATUS2     DATA+1
#define  CONTROL2    DATA+2

//  LPT3 register address
#define  DATA3       0x3BC
#define  STATUS3     DATA+1
#define  CONTROL3    DATA+2





#define  PORT_ABC_out        0x80
#define  PORT_A_in_BC_out    0x90
#define  PORT_ABC_in         0x9B

int   DATA, STATUS, CONTROL;

typedef  unsigned char byte;


   void  init_8255_mode(byte mode8255);
   void write_data_to_PORT_A(byte );
   void write_data_to_PORT_B(byte );
   void write_data_to_PORT_C(byte );
   void write_62256(byte );
   byte read_62256();
   void  change_rom_mode();

   byte   low_address = 0x00;
   byte   high_address = 0x00;
   byte   memory_data;
   int    delay_time = 0;

void main(int argc, char *argv[])
{


   int   i, j;
   int   ch;
   FILE  *fp;
   int ram_data;


   if(argc<2)
   {
      printf("MyRom  Version  1.0  made by Han Dong Soo in KOJE-DO          \n");
      printf("Syntex:  myrom [option] filename                              \n");
      printf("option:  -L1 (Printer port address is 0x378. This is default.)\n");
      printf("         -L2 (Printer port address is 0x278)                  \n");
      printf("         -L3 (Printer port address is 0x3BC)                  \n");
      printf("filename must follow its suffix.                              \n");
      printf("Example: myrom v40.bin                                        \n");
      printf("         myrom -L2 v40.bin                                    \n");
      exit(-1);
   }

   if(argc == 2) //printer port address is default setting.
   {
      DATA = DATA1;
      STATUS = STATUS1;
      CONTROL = CONTROL1;

      if(( fp = fopen(argv[1], "rb")) ==NULL)
      {
	printf("File open error!\n");
	exit(-1);
      }
   }

   else if(argc == 3)
   {
      if( strcmp(argv+1, "-L1") == 0)
      {  DATA = DATA1;
	 STATUS = STATUS1;
	 CONTROL = CONTROL1;
      }

      else if( strcmp(argv+1, "-L2") == 0)
      {  DATA = DATA2;
	 STATUS = STATUS2;
	 CONTROL = CONTROL2;
      }

      else if( strcmp(argv+1, "-L3") == 0)
      {  DATA = DATA3;
	 STATUS = STATUS3;
	 CONTROL = CONTROL3;
      }


      if(( fp = fopen(argv[2], "rb")) ==NULL)
      {
	printf("File open error!\n");
	exit(-1);
      }

   }




   // CONTROL      x     x     x     x    G   /WR  /RD   NU
   // 373 DATA     A1    A0  /rom   /2G  /1G  /WE  /OE  /CS


   clrscr();

   low_address  = 0;
   high_address = 0;

   memory_data = 0x0;

   printf("Start data downloading.\n");

   init_8255_mode(PORT_ABC_out);

   for( i=0; i<128; i++)
   {
      for( j=0; j<256; j++)
      {
	  if(( ch=fgetc(fp)) == EOF)  ch = 0;

	  memory_data = ch;

	  write_62256(memory_data);

	  low_address++;
      }

      high_address++;
      printf(".");
  }

printf("Data download complete.\n");

///////////////////////////////////////
//verify 62256 ram data
///////////////////////////////////////

printf("\nStart verifing.\n");

fseek(fp, 0, SEEK_SET);

low_address  = 0x00;
high_address = 0x00;

init_8255_mode(PORT_A_in_BC_out);

for( i=0; i<128; i++)
{
    for( j=0; j<256; j++)
    {
      if(( ch=fgetc(fp)) == EOF)  ch = 0;

      ram_data = read_62256();
      if( ch != ram_data )
      {
	 printf("Error in verifing!\n");
	 printf("Memory address  0x%02X%02X  file_data:%02X  ram_data:%02X\n", low_address, high_address,  ch, read_62256());
	 printf("Please check the device and retry again.\n");
	 exit(-1);
      }
      low_address++;
    }

    high_address++;
    printf(".");
}

   printf("Verify OK!\n");

   change_rom_mode();

   fclose(fp);

   printf("\nThank you for using MyRom. \n");

//   getch();
}


   // CONTROL      x     x     x     x    G   /WR  /RD   NU
   // 373 DATA     A1    A0  /rom   /2G  /1G  /WE  /OE  /CS





void  init_8255_mode(byte mode8255)
{
   outportb(DATA, 0x87);   // CON resister, rom disable , 2G high, 1G low,
			   // WE disable, OE disable, CS disable

   
   outportb(CONTROL, 0x0E^ 0x0B); // ensable 373, WR disable, RD disable
                                  // select 8255 control resister
   

   outportb(DATA, 0xF7);   // CON resister, rom disable , 2G high, 1G low,
			   // WE disable, OE disable, CS disable
   

   outportb(CONTROL, 0x06^ 0x0B); // disable 373, WR disable, RD disable
   
   //delay(delay_time);
   outportb(CONTROL, 0x02^ 0x0B);// disable 373, WR ensable, RD disable
   
   //delay(delay_time);

   outportb(DATA,  mode8255);
   //delay(delay_time);

   outportb(CONTROL, 0x06^ 0x0B); // disable 373, WR disable, RD disable
   
   //delay(delay_time);
}





void  write_data_to_PORT_C(byte data)
{
   outportb(DATA, 0xB7); // PORT C, rom disable , 2G high, 1G low,
				    // WE disable, OE disable, CS disable
   
   outportb(CONTROL, 0x0E^ 0x0B); // ensable 373, WR disable, RD disable		
                        	  // select 8255 control resister   
   //delay(delay_time);
   outportb(CONTROL, 0x06^ 0x0B); // disable 373, WR disable, RD disable
   
   //delay(delay_time);
   outportb(CONTROL, 0x02^ 0x0B); // disable 373, WR ensable, RD disable
   
   //delay(delay_time);

   outportb(DATA,  data);
   //delay(delay_time);

   outportb(CONTROL, 0x06^ 0x0B);// disable 373, WR disable, RD disable
   
   //delay(delay_time);
}



void  write_data_to_PORT_B(byte data)
{
   outportb(DATA, 0x77);  // PORT B, rom disable , 2G high, 1G low,
			  // WE disable, OE disable, CS disable
   
   outportb(CONTROL, 0x0E ^ 0x0B); // ensable 373, WR disable, RD disable		
                                   // select 8255 control resister
   
   //delay(delay_time);
   outportb(CONTROL, 0x06 ^ 0x0B); // disable 373, WR disable, RD disable
   
   //delay(delay_time);
   outportb(CONTROL, 0x02 ^ 0x0B); // disable 373, WR ensable, RD disable
   
   //delay(//delay_time);

   outportb(DATA,  data);
   //delay(//delay_time);

   outportb(CONTROL, 0x06 ^ 0x0B); // disable 373, WR disable, RD disable
   
   //delay(//delay_time);
}


void  write_data_to_PORT_A(byte data)
{
   outportb(DATA, 0x37); // PORT A, rom disable , 2G high, 1G low,
			 // WE disable, OE disable, CS disable
   
   outportb(CONTROL, 0x0E ^ 0x0B);// ensable 373, WR disable, RD disable	
                		  // select 8255 control resister
   
   //delay(delay_time);
   
   outportb(CONTROL, 0x02 ^ 0x0B); // disable 373, WR ensable, RD disable
   
   //delay(//delay_time);

   outportb(DATA,  data);
   //delay(//delay_time);

   outportb(CONTROL, 0x06 ^ 0x0B); // disable 373, WR disable, RD disable
   
   //delay(//delay_time);
}






void write_62256(byte rom_data)
{


   
   write_data_to_PORT_C(high_address);
   write_data_to_PORT_B(low_address);
   write_data_to_PORT_A(0xFF);



///////////////////////////////////////////////////////////////
//   CONTROL      x     x     x     x    G   /WR  /RD   NU   //
//   373 DATA     A1    A0  /rom   /2G  /1G  /WE  /OE  /CS   //
///////////////////////////////////////////////////////////////


//write data to 62256

   outportb(DATA, 0x37); // PORT A, rom disable , 2G high, 1G low,
			 // WE disable, OE disable, CS disable
   
   outportb(CONTROL,  0x0E ^ 0x0B);  // 373 enable, WR disable, RD disable
   
    outportb(DATA,  0x36); // PORT A, rom disable , 2G high,
			   // 1G low, WE disable, OE disable, CS enable
  

   outportb(DATA, 0x32);  // PORT A, rom disable , 2G high,
		          // 1G low, WE enable, OE disable, CS enable
   

// 8255 PORT A data output
   
   outportb(CONTROL,  0x06 ^ 0x0B);  // 373 disable, WR disable, RD disable
   
   //delay(//delay_time);
   outportb(CONTROL,  0x02 ^ 0x0B); // 373 disable, WR enable, RD disable
   
   //delay(//delay_time);
   outportb(DATA, rom_data);
   //delay(//delay_time);
   outportb(CONTROL,  0x06 ^ 0x0B); // 373 disable, WR disable, RD disable
   
   //delay(//delay_time);
  
   outportb(DATA, 0x36); // PORT A, rom disable , 2G high,
			 // 1G low, WE disable, OE disable, CS enable
   
   outportb(CONTROL,  0x0E ^ 0x0B);  // 373 enable, WR disable, RD disable
   
   outportb(DATA,  0x37); // PORT A, rom disable , 2G high,
			  // 1G low, WE disable, OE disable, CS disable
   
   outportb(CONTROL,  0x06 ^ 0x0B); // 373 disable, WR disable, RD disable
   
// end of writing data to 62256
}



byte read_62256()
{
   byte    read_rom_data;

   //delay(//delay_time);
   write_data_to_PORT_B(low_address);
   write_data_to_PORT_C(high_address);

///// Read from 62256
   outportb(DATA, 0x37); // PORT A, rom disable , 2G high, 1G low,
			 // WE disable, OE disable, CS disable
   

   outportb(CONTROL, 0x0E ^ 0x0B); // 373 enable, WR disable, RD disable
   
   //delay(//delay_time);

   outportb(DATA, 0x36); // PORT A, rom disable , 2G high, 1G low,
			 // WE disable, OE disable, CS enable
   

   outportb(DATA, 0x34); // PORT A, rom disable , 2G high, 1G low,
			 // WE disable, OE enable, CS enable
   

   outportb(CONTROL, 0x06 ^ 0x0B);  // 373 disable, WR disable, RD disable
   
   //delay(delay_time);

   outportb(CONTROL, 0x24 ^ 0x0B);  // enable bi-diretion,
				    // 373 disable, WR disable, RD enable
   
   //delay(delay_time);

   read_rom_data=inportb(DATA);
   //printf( ".. %02X", read_rom_data);


   outportb(CONTROL, 0x06 ^ 0x0B); // 373 disable, WR disable, RD disable
   
   outportb(CONTROL, 0x0E ^ 0x0B);  // 373 enable, WR disable, RD disable
   
   outportb(DATA, 0x37); // PORT A, rom disable , 2G high, 1G low,
			 // WE disable, OE disable, CS disable
   
   outportb(CONTROL, 0x06 ^0x0B);// 373 disable, WR disable, RD disable
   
   return read_rom_data;
}



void  change_rom_mode()
{
    init_8255_mode(PORT_ABC_in);

    outportb(DATA, 0xCF);
    outportb(CONTROL, 0x0E ^ 0x0B);
    outportb(DATA, 0xCF);
    outportb(CONTROL, 0x06 ^ 0x0B);
}